<?PHP
/*
Plugin Name: Youtube Feeder
Plugin URI: http://andrewensley.com/youtube-feeder-wordpress-plugin/
Version: 1.0.2
Author: Andrew Ensley
Author URI: http://andrewensley.com/
Description: Allows you to embed a dynamic Youtube video feed anywhere on your Wordpress blog.
License: GPLv3
*/

if (!class_exists('YoutubeFeeder'))
{
	class YoutubeFeeder
	{
		// Properties
		private $adminOptionsName = 'YoutubeFeederAdminOptions',
			$cacheOptionName = 'YoutubeFeederCache',
			$options = array(),
			$settings = array('user', 'limit', 'width', 'height', 'dateformat', 'datelocation', 'style', 'cache', 'thumbnail'),
			$ytCount = 0;
		// Constructor
		public function YoutubeFeeder()
		{
			// Make sure this is an instantiated object, and not a static access of this constructor
			// why on earth someone would do that.... who knows, but this code is safe
			if(isset($this))
			{
				// Get options from DB
				$this->options = get_option($this->adminOptionsName);
				//Add the admin menu item
				add_action('admin_menu', array(&$this, 'youtubefeeder_ap'));
				// Add init function to run on activation of the plugin
				add_action('activate_' . plugin_basename(__FILE__),  array(&$this, 'init'));
				// Add CSS and JS
				add_action('wp_head', array(&$this, 'addHeaderCode'));
				if (function_exists('wp_enqueue_script'))
				{
					wp_enqueue_script('YoutubeFeeder_jquery_youtubeplaylist',
						WP_PLUGIN_URL . '/youtube-feeder/js/jquery.youtubeplaylist.js',
						array('jquery'),
						'1.1');
				}
				// Add Youtube Shortcode
				add_shortcode('youtubefeeder', array(&$this, 'youtubeFeederShortcode'));
				// Add Youtube Shortcode
				add_shortcode('youtubefeederpublisheddate', array(&$this, 'getPublishedDate'));
			}
		}
		// This runs when the plugin is activated
		public function init()
		{
			// Update options in DB with all other defaults if there are any new settings.
			$this->setAdminOptions();
		}
		
		public function addHeaderCode()
		{
			echo '<link type="text/css" rel="stylesheet" href="', WP_PLUGIN_URL, '/youtube-feeder/css/style.css" />', "\n";
		}

		public function youtubeFeederShortcode($atts)
		{
			$this->ytCount++;
			$defaults = array();
			foreach($this->settings as $s)
			{
				$defaults[$s] = $this->options[$s];
			}
			extract(shortcode_atts($defaults, $atts));
			$data = $this->getYoutubeData($user, $cache);
			if(!$data)
			{
				return '<div style="border:1px solid red;margin:5px 5%;font-weight:bold;">Unable to get Youtube feed.  '
					. 'Did you supply the correct user?</div>';
			}
			$content = '<div class="youtubeFeeder youtubeFeeder' . $style . '" style="min-height:' . $height . 'px;">';
			switch($style)
			{
				case 'playlist':
					$this->youtubePlaylistStyle($data, $content, $limit, $width, $height, $dateformat, $datelocation, $thumbnail, $this->ytCount);
					break;
				case 'list':
				default:
					$this->youtubeListStyle($data, $content, $limit, $width, $height, $dateformat, $datelocation, $this->ytCount);
					break;
			}
			$content .= '</div>';
			return $content;
		}
		
		private function getYoutubeData($user, $cache)
		{
			if(!$entries = $this->getCachedYoutubeData($user, $cache))
			{
				$data = @json_decode(file_get_contents('http://gdata.youtube.com/feeds/api/users/' . $user . '/uploads?alt=json'), TRUE);
				$entries = array();
				foreach($data['feed']['entry'] as $vid)
				{
					$url = $vid['media$group']['media$content'][0]['url'];
					$url = substr($url, 0, stripos($url, '?'));
					$entries[] = array(
							'url' => $url,
							'title' => $vid['title']['$t'],
							'date' => $vid['published']['$t'],
							'content' => $vid['content']['$t']
						);
				}
				if(count($entries) > 0)
				{
					$this->cacheYoutubeData($user, $entries);
				}
				else
				{
					$entries = false;
				}
			}
			return $entries;
		}
		
		private function getCachedYoutubeData($user, $cache)
		{
			$youtubeCache = get_option($this->cacheOptionName);
			if(empty($youtubeCache) || empty($youtubeCache[$user]) || empty($youtubeCache[$user.'-date'])
				|| strtotime($youtubeCache[$user.'-date']) < strtotime('-' . $cache))
			{
				return false;
			}
			else
			{
				return $youtubeCache[$user];
			}
		}
		
		private function cacheYoutubeData($user, $data)
		{
			// Load the cache stored in the db
			$youtubeCache = get_option($this->cacheOptionName);
			if(empty($youtubeCache))
			{
				$youtubeCache = array();
			}
			$youtubeCache[$user] = $data;
			$youtubeCache[$user.'-date'] = date('Y-m-d H:i:s');
			// Store updated options back in the database.
			update_option($this->cacheOptionName, $youtubeCache);
		}
		
		private function youtubePlaylistStyle($data, &$content, $limit, $width, $height, $dateformat, $datelocation, $thumbnail, $ytCount)
		{
			$content .= '<div class="youtubeVideo" id="youtubeFeederPlayer' . $ytCount . '"></div>'
				. '<ul class="youtubeFeederPlaylistList" id="youtubeFeederPlaylist' . $ytCount . '">';
			$counter = 0;
			foreach($data as $vid)
			{
				$date = '<span class="youtubeDate' . $datelocation . '">'
					. date($dateformat, strtotime($vid['date'])) . '</span>';
				$content .= '<li><a href="' . $vid['url'] . '">'
					. ($thumbnail == 'true' ? '<img alt="' . $vid['title'] . '" src="http://img.youtube.com/vi/'
					.  substr($vid['url'], stripos($vid['url'], '/v/') + 3) . '/2.jpg">' : '')
					. ($datelocation == 'beforetitle' ? $date : '') . '<span class="youtubeTitle">' . $vid['title'] . '</span>'
					. ($datelocation == 'aftertitle' ? $date : '') . '</a></li>';
				$counter++;
				if($counter == $limit)
				{
					break;
				}
			}
			$content .= '</ul><script>jQuery("#youtubeFeederPlaylist' . $ytCount . '").ytplaylist({playerHeight:"'
				. $height . '",playerWidth:"' . $width . '",holderId:"youtubeFeederPlayer' . $ytCount . '"' . '});</script>';
		}

		private function youtubeListStyle($data, &$content, $limit, $width, $height, $dateformat, $datelocation, $ytCount)
		{
			$counter = 0;
			foreach($data as $vid)
			{
				$date = '<span class="youtubeDate' . $datelocation . '">'
					. date($dateformat, strtotime($vid['date'])) . '</span>';
				$content.= ($datelocation == 'beforevideo' ? $date : '')
					. '<div class="youtubeVideo"><object width="' . $width . '" height="' . $height . '">'
					. '<param name="movie" value="' . $vid['url'] . '"></param>'
					. '<param name="allowfullscreen" value="true"></param>'
					. '<param name="allowscriptaccess" value="always"></param>'
					. '<embed src="' . $vid['url'] . '" type="application/x-shockwave-flash" allowscriptaccess="always"'
						. ' allowfullscreen="true" width="' . $width . '" height="' . $height . '"></embed></object></div>'
					. '<div class="youtubeTitle">' . ($datelocation == 'beforetitle' ? $date : '')
					. $vid['title'] . ($datelocation == 'aftertitle' ? $date : '') . '</div>'
					. '<div class="youtubeContent">' . $vid['content'] . '</div>' . "\n";
				$counter++;
				if($counter == $limit)
				{
					break;
				}
			}
		}
		
		public function getPublishedDate($atts)
		{
			$defaults = array();
			foreach($this->settings as $s)
			{
				$defaults[$s] = $this->options[$s];
			}
			extract(shortcode_atts($defaults, $atts));
			$data = $this->getYoutubeData($user, $cache);
			if(!$data)
			{
				return '';
			}
			else
			{
				return date($dateformat, strtotime($data[0]['date']));
			}
		}

		// Adds the options page link for Youtube Feeder
		public function YoutubeFeeder_ap()
		{
			if(isset($this))
			{
				if(function_exists('add_options_page'))
				{
					add_options_page('YoutubeFeeder', 'YoutubeFeeder', 9, basename(__FILE__), array(&$this, 'printAdminPage'));
				}
			}
		}
		public function setAdminOptions()
		{
			// Default values for configuration options
			$youtubeAdminOptions = array(
					'user' => '',
					'limit' => 5,
					'height' => 385,
					'width' => 480,
					'style' => 'list',
					'dateformat' => 'n/d/y g:ia',
					'datelocation' => 'aftertitle',
					'cache' => '1 days',
					'thumbnail' => 'true'
				);
			// Load options stored in the Database
			$youtubeOptions = get_option($this->adminOptionsName);
			if(!empty($youtubeOptions))
			{
				// Overwrite defaults with options retrieved from the database
				foreach($youtubeOptions as $key => $option)
				{
					$youtubeAdminOptions[$key] = $option;
				}
			}
			// Store updated options back in the database.
			update_option($this->adminOptionsName, $youtubeAdminOptions);
		}
		//Prints out the admin page
		public function printAdminPage()
		{
			$this->addHeaderCode();
			if (function_exists('wp_enqueue_script'))
			{
				wp_enqueue_script('YoutubeFeeder_jquery_youtubeplaylist',
					WP_PLUGIN_URL . '/youtube-feeder/js/jquery.youtubeplaylist.js',
					array('jquery'),
					'1.1');
			}
			// Clear Cache button was pressed
			if(isset($_POST['YoutubeFeeder_clearcache']))
			{
				if(delete_option($this->cacheOptionName))
				{
					?><div class="updated"><p><strong><?php
						_e('Youtube Feeder Cache Cleared', 'YoutubeFeeder');
					?></strong></p></div><?php
				}
				else
				{
					?><div class="error"><p><strong><?php
						_e('Unable to clear Youtube Feeder Cache', 'YoutubeFeeder');
					?></strong></p></div><?php
				}
			}
			// The form was submitted
			if(isset($_POST['update_YoutubeFeederSettings']))
			{
				foreach($this->settings as $s)
				{
					if($s == 'cache')
					{
						if(isset($_POST['cachenum']))
						{
							switch($_POST['cacheunit'])
							{
								case 'minutes':
									$unit = 'minutes';
									break;
								case 'hours':
									$unit = 'hours';
									break;
								case 'weeks':
									$unit = 'weeks';
									break;
								case 'months':
									$unit = 'months';
									break;
								case 'days':
								default:
									$unit = 'days';
									break;
							}
							$tnum = $_POST['cachenum'];
							$timeperiod = ($tnum == 0 ? '0' : $tnum . ' ' . $unit);
							$this->options['cache'] = $timeperiod;
						}
					}
					elseif(isset($_POST[$s]))
					{
						$this->options[$s] = $_POST[$s];
					}
				}
				// Store options in the database
				if(update_option($this->adminOptionsName, $this->options))
				{
					?><div class="updated"><p><strong><?php
						_e('Settings Updated.', 'YoutubeFeeder');
					?></strong></p></div><?php
				}
				else
				{
					?><div class="error"><p><strong><?php
						_e('Unable to update settings.', 'YoutubeFeeder');
					?></strong></p></div><?php
				}
			}
			?>
			<div class=wrap>
				<form method="post" action="<?php echo $_SERVER['REQUEST_URI']; ?>">
					<h2>Youtube Feeder Plugin</h2>
					<br /><strong>The following settings are merely defaults.<br />
					All settings can be overriden by shortcode attributes or parameters passed to the function call.</strong><br/>
					<small>For FAQ and support, refer to the <a href="http://andrewensley.com/projects/youtube-feeder-wordpress-plugin/" title="Youtube Feeder Wordpress Plugin | AndrewEnsley.com" target="_blank">Youtube Feeder Plugin page.</a></small>
					<hr/>
					<h3>Youtube Username</h3>
					<input type="text" name="user" value="<?php
						echo $this->options['user'];
					?>" size="30"/>
					<hr/>
					<h3>Limit</h3>
					<input type="text" name="limit" value="<?php
						echo $this->options['limit'];
					?>" size="5"/><br/>
					The maximum number of videos to display.
					<hr/>
					<h3>Size</h3>
					Width: <input type="text" name="width" value="<?php
						echo $this->options['width'];
					?>" size="4"/> x Height: <input type="text" name="height" value="<?php
						echo $this->options['height'];
					?>" size="4"/><br/>
					The size of the video player.
					<hr/>
					<h3>Style</h3>
					<input type="radio" name="style" value="list" id="style-list"<?php
						echo ($this->options['style'] == 'list' ? ' checked' : '');
					?> /><label for="style-list"><strong>List</strong><br/>
					Preview:
					<div style="height:350px;overflow:auto;border:1px solid black;"><?php
						echo do_shortcode('[youtubefeeder user="flamadiddle86" limit="2" style="list" width="335" height="275" cache="1 weeks"]');
					?></div></label><br/>
					<input type="radio" name="style" value="playlist" id="style-playlist"<?php
						echo ($this->options['style'] == 'playlist' ? ' checked' : '');
					?> /><label for="style-playlist"><strong>Play List</strong><br/>
					Preview:
					<div style="height:330px;overflow:auto;border:1px solid black;"><?php
						echo do_shortcode('[youtubefeeder user="flamadiddle86" limit="5" style="playlist" width="335" height="275" cache="1 weeks"]');
					?></div></label>
					<hr/>
					<h3>Show Thumbnails</h3>
					<input type="radio" name="thumbnail" value="true" id="thumbnail-true"<?php
						echo ($this->options['thumbnail'] == 'true' ? ' checked' : '');
					?> /><label for="thumbnail-true">Yes</label><br/>
					<input type="radio" name="thumbnail" value="false" id="thumbnail-false"<?php
						echo ($this->options['thumbnail'] == 'false' ? ' checked' : '');
					?> /><label for="thumbnail-false">No</label><br/>
					Only affects the "playlist" style.  This is whether or not to show video thumbnails in the play list.
					<hr/>
					<h3>Date Format</h3>
					<input type="text" name="dateformat" value="<?php
						echo $this->options['dateformat'];
					?>" size="20"/> <?php
						echo '<strong>', date($this->options['dateformat']),
							'</strong> &lt;--Current format (\'', $this->options['dateformat'], '\')';
					?><br/>
					<small>
						For an explanation of available date formats, see the
						<a href="http://php.net/date" title="PHP date() Function" target="_blank">doc page for the PHP date() function</a>
					</small>
					<hr/>
					<h3>Date Location</h3>
					<input type="radio" name="datelocation" value="beforevideo" id="datelocation-beforevideo"<?php
						echo ($this->options['datelocation'] == 'beforevideo' ? ' checked' : '');
					?> /><label for="datelocation-beforevideo">Before Video</label><br/>
					<input type="radio" name="datelocation" value="beforetitle" id="datelocation-beforetitle"<?php
						echo ($this->options['datelocation'] == 'beforetitle' ? ' checked' : '');
					?> /><label for="datelocation-beforetitle">Before Title</label><br/>
					<input type="radio" name="datelocation" value="aftertitle" id="datelocation-aftertitle"<?php
						echo ($this->options['datelocation'] == 'aftertitle' ? ' checked' : '');
					?> /><label for="datelocation-aftertitle">After Title</label><br/>
					<input type="radio" name="datelocation" value="nowhere" id="datelocation-nowhere"<?php
						echo ($this->options['datelocation'] == 'nowhere' ? ' checked' : '');
					?> /><label for="datelocation-nowhere">Don't display date</label><br/>
					<hr/>
					<h3>Cache Time Period</h3>
					<input type="text" name="cachenum" value="<?php
						echo substr($this->options['cache'], 0, stripos($this->options['cache'], ' '));
						$unit = substr($this->options['cache'], (stripos($this->options['cache'], ' ') + 1));
					?>" size="5"/><select name="cacheunit">
						<option<?php
							echo ($unit == 'minutes' ? ' selected' : '');
						?>>minutes</option>
						<option<?php
							echo ($unit == 'hours' ? ' selected' : '');
						?>>hours</option>
						<option<?php
							echo ($unit == 'days' ? ' selected' : '');
						?>>days</option>
						<option<?php
							echo ($unit == 'weeks' ? ' selected' : '');
						?>>weeks</option>
						<option<?php
							echo ($unit == 'months' ? ' selected' : '');
						?>>months</option>
					</select><br/>
					This plugin can cache the data in the Youtube feed to reduce server load.<br/>Enter "0" (without quotes) to disable
					caching and always serve the most up-to-date feed.<div class="submit">
					<input type="submit" name="YoutubeFeeder_clearcache" value="<?php
						_e('Clear Cache Now', 'YoutubeFeeder');
					?>" /></div>
					<hr/>
					<div class="submit">
					<input type="submit" name="update_YoutubeFeederSettings" value="<?php
						_e('Update Settings', 'YoutubeFeeder');
					?>" /></div>
				</form>
			</div>
			<?php
		}//End function printAdminPage()
	}
}//End Class YoutubeFeeder

if(class_exists('YoutubeFeeder'))
{
	$YoutubeFeeder = new YoutubeFeeder();
}

if(!function_exists('embed_youtube_feed'))
{
	function embed_youtube_feed($options = array())
	{
		$shortcode = '[youtubefeeder';
		foreach($options as $key=>$val)
		{
			$shortcode .= ' ' . $key . '"' . $val . '"';
		}
		$shortcode .= ']';
		echo do_shortcode($shortcode);
	}
}

if(!function_exists('get_youtube_published_date'))
{
	function get_youtube_published_date($options = array())
	{
		$shortcode = '[youtubefeeder-publisheddate';
		foreach($options as $key=>$val)
		{
			$shortcode .= ' ' . $key . '"' . $val . '"';
		}
		$shortcode .= ']';
		echo do_shortcode($shortcode);
	}
}

?>